//
// Implementation de la classe DataMergerGroup
//

#ifndef DataMergerGroup_C
#define DataMergerGroup_C

#include "DataMergerGroup.h"
#include "FParamBrutSimple.h"
#include "FParamBrutVector.h"
#include "Riostream.h"

//ClassImp(DataMergerGroup)

//____________________________________________________________________
DataMergerGroup::DataMergerGroup(void):TNamed()
{
//
// Constructeur
//
chanToNs=7.8125e-3;
tsMax=-1.;
nbGroupes=0;
}

//____________________________________________________________________
DataMergerGroup::~DataMergerGroup(void)
{
//
// Destructeur
//
}

//____________________________________________________________________
void DataMergerGroup::AddTriggerParam(FParam *px, FParam *TSx, 
                                 FParam *&vOut, FParam *&vOutTS)
{
//
// Ajout d'un parametre Trigger et de son timeStamp
//
pTrigger.push_back(px);
pTriggerTS.push_back(TSx);
// On genere les parametres de sortie associes
if(!vOut)
 {
 cout << "Creation du parametre associe a " << px->GetName() << endl;
 vOut=new FParamBrutVector();
 vOut->SetName(Form("%s_%s",this->GetName(),px->GetName()));
 vOut->SetNbValuesMax(1);
 }
if(!vOutTS)
 {
 cout << "Creation du parametre associe a " << TSx->GetName() << endl;
 vOutTS=new FParamBrutVector();
 vOutTS->SetName(Form("%s_%s",this->GetName(),TSx->GetName()));
 vOutTS->SetNbValuesMax(1);
 }
vOutTrig.push_back(vOut);
vOutTrigTS.push_back(vOutTS);
valTS *v=new valTS();
vPtrig.push_back(v);
nt=pTrigger.size();
nbTrig.resize(nt);
}

//____________________________________________________________________
void DataMergerGroup::AddSlaveParam(FParam *ps, FParam *TSs,
                               FParam *&vOut, FParam *&vOutTS)
{
//
// Ajout des parametres esclaves
//
pSlave.push_back(ps);
pSlaveTS.push_back(TSs);
// On genere les parametres de sortie associes
if(! vOut)
 {
 cout << "Creation du parametre associe a " << ps->GetName() << endl;
 vOut=new FParamBrutVector();
 vOut->SetName(Form("%s_%s",this->GetName(),ps->GetName()));
 vOut->SetNbValuesMax(1);
 }
if(! vOutTS)
 {
 cout << "Creation du parametre associe a " << TSs->GetName() << endl;
 vOutTS=new FParamBrutVector();
 vOutTS->SetName(Form("%s_%s",this->GetName(),TSs->GetName()));
 vOutTS->SetNbValuesMax(1);
 }
vOutSlave.push_back(vOut);
vOutSlaveTS.push_back(vOutTS);
valTS *v=new valTS();
vPslave.push_back(v);
ns=pSlave.size();
}

//____________________________________________________________________
Double_t DataMergerGroup::ComputeTriggerTime(Int_t ig)
{
 // 
 // On va calcuer le temps de reference du trigger: on va prendre le temps moyen de tous les triggers
 // dans la fenetre de decision
 //
 Double_t timeRef=0;
 Int_t nval=0;
 Double_t numG=vGroupe[ig][0];
 for(Int_t it=0;it<nt;it++)
  {
  valTS *vec=vPtrig[it];
  Int_t nbt=(*vec).size();
  // On va maintenant scanner les valeurs presentes dans ce tableau
  Int_t i=0;
  while(i < nbt)
   {
   Double_t grp=(*vec)[i][2];  // on recupere le numero de groupe
   if(grp == numG) // s'il appartient au groupe
    {
    nval++; // on incremente le compteur
    timeRef+=(*vec)[i][1];
    }
   i++;
   }
  }
 if(nval > 0) timeRef/=nval;
 return timeRef;
}

//____________________________________________________________________
Bool_t DataMergerGroup::Decision(Int_t ig)
{
 // 
 // On va prendre la decition sur le nombre de triggers dans la fenetre
 //
 Bool_t ok=kTRUE;
 Double_t numG=vGroupe[ig][0];
 for(Int_t it=0;it<nt;it++)
  {
  nbTrig[it]=0;
  valTS *vec=vPtrig[it];
  Int_t nbv=(*vec).size();
  for(Int_t i=0;i<nbv;i++)
   {
   if((*vec)[i][2] == numG) 
    {
    nbTrig[it]++;
    }
   }
  }

 ok=(nbTrig[0]==1 && nbTrig[1]==1);
 
 return ok;
}

//____________________________________________________________________
void DataMergerGroup::MergeData(Int_t ig)
{
 // On va collecter les donnees qui sont dans la fenetre de collection et 
 // ajuster les valeurs des parametres presents dans vOutParam
 //
 //cout << "Entree dans MergeData() " << endl;
 Double_t timeRef=ComputeTriggerTime(ig);
 Double_t tcMin=timeRef+cDelay;
 Double_t tcMax=tcMin+cGate;
 Double_t numG=vGroupe[ig][0];
 //cout << "Time Ref : "  << timeRef << endl;
 //
 // On collecte les donnees des triggers
 //
 for(Int_t it=0;it<nt;it++)
  {
  valTS *vec=vPtrig[it];
  Int_t nbt=(*vec).size();
  // On va maintenant scanner les valeurs presentes dans ce tableau
  Int_t i=0;
  Int_t nval=0;
  Double_t ng=0;
  //cout << "Val trigger : " << it << endl;
  while(i < nbt && ng <= numG)
   {
   ng=(*vec)[i][2]; // on recupere le numero de groupe
   //cout << "          " << it<< " , " << i << " : " << ts << endl;
   if(ng == numG) // s'il appartient au groupe
    {
    //cout << "          Ok : " <<vOutParam[iOut] << " / " << vOutParam[iOut+1] << endl;
    vOutTrig[it]->SetValue((*vec)[i][0],nval);
    vOutTrigTS[it]->SetValue((*vec)[i][1],nval);
    nval++;
    // On efface cette valeur du tableau
    //cout << "On efface l'element " << i << endl;
    vec->erase((*vec).begin()+i);
    nbt--;
    //cout << "fin ajout "<< endl; 
    }
   else
    {
    i++;
    }
   }
  }
 //
 // On collecte les donnees des esclaves
 //
 for(Int_t is=0;is<ns;is++)
  {
  valTS *vec=vPslave[is];
  Int_t nbs=(*vec).size();
  // On va maintenant scanner les valeurs presentes dans ce tableau
  Int_t i=0;
  Double_t ts=-1.;
  Int_t nval=0;
  //cout << "Val slave : " << is << endl;
  while(i < nbs && ts <= tcMax)
   {
   ts=(*vec)[i][1]; // on recupere le TS
   //cout << "          " << is<< " , " << i << " : " << ts << endl;
   if(ts >= tcMin && ts <= tcMax) // s'il est dans la fenetre de collection
    {
    //cout << "          Ok " << endl;
    vOutSlave[is]->SetValue((*vec)[i][0],nval);
    vOutSlaveTS[is]->SetValue((*vec)[i][1],nval);
    nval++;
    // On efface cette valeur du tableau
    vec->erase((*vec).begin()+i);
    nbs--;
    }
   else if(i==0 && ts < tcMin)
    {
    //cout << "          TS trop bas" << endl;
    vec->erase((*vec).begin()+i);
    nbs--;
    }
   else
    {
    i++;
    }
   }
  }
 //cout << "Sortie de MergeData() " << endl;
}

//____________________________________________________________________
void DataMergerGroup::Process(void)
{

 //cout << "Process de " << GetName() << endl;
 
 //
 // On ajoute une valeur a vPtrig si necessaire
 //
 for(Int_t it=0;it<nt;it++)
  {
  if(pTrigger[it]->IsOn() && pTriggerTS[it]->IsOn())
   {
   Double_t *val=new Double_t[3];
   val[0]=pTrigger[it]->GetValue();             // valeur
   val[1]=pTriggerTS[it]->GetValue()*chanToNs;  // TS
   val[2]=-1;                                   // numero de groupe
   //
   // On va determiner son numero de groupe 
   //
   Int_t ng=vGroupe.size();
   for(Int_t ig=0;ig<ng && val[2]<0;ig++)
    {
    Double_t *vec=vGroupe[ig];
    Double_t tdMin=vec[1];
    Double_t tdMax=vec[2];
    if(val[1] >=tdMin && val[1] <=tdMax) // Si TS est dans la fenetre de decision du groupe
     {
     val[2]=vec[0]; // on affecte le numero du groupe
     }
    }
   if(val[2]<0) // aucun groupe n'a ete trouve
    {
    nbGroupes++;      // on en cree un nouveau
    val[2]=nbGroupes; // et on affecte la valeur de groupe
    Double_t *gVal=new Double_t[3];
    gVal[0]=nbGroupes; // numro du groupe
    gVal[1]=val[1];    // tMin
    gVal[2]=val[1]+dGate; // tMax
    vGroupe.push_back(gVal); // on l'ajoute au tableau des groupes
    }
   if(val[1] > tsMax) tsMax=val[1];
   vPtrig[it]->push_back(val);
   }
  }
 
 //
 // On ajoute les valeurs a vPslave si necessaire
 //
 for(Int_t is=0;is<ns;is++)
  {
  if(pSlave[is]->IsOn() && pSlaveTS[is]->IsOn())
   {
   Double_t *val=new Double_t[2];
   val[0]=pSlave[is]->GetValue();
   val[1]=pSlaveTS[is]->GetValue()*chanToNs;
   if(val[1] > tsMax) tsMax=val[1];
   vPslave[is]->push_back(val);
   }
  }
 
 //PrintValues();
 
 // On va maintenant prendre la decision de traiter ou non un groupe
 for(Int_t ig=0;ig<vGroupe.size();ig++)
  {
  Double_t tgMax = vGroupe[ig][2];
  if(tsMax > tgMax) // tous les parametre triggers possibles sont passes
   {
   if(Decision(ig)) // si le groupe est valide
    {
    Double_t timeRef=ComputeTriggerTime(ig);
    if(tsMax > timeRef+cDelay+cGate) // toutes les donnees possibles ont ete vues
     {
     MergeData(ig);
     }
    }
   else // si le groupe n'est pas valide, on le detruit
    {
    Double_t numG=vGroupe[ig][0];
    Double_t tcMin=ComputeTriggerTime(ig)+cDelay;
    //cout << "Destruction des triggers du groupe " << ig << " / " << numG << " / " << tcMin << endl;
    vGroupe.erase(vGroupe.begin()+ig);
    // On efface les donnee trigger appartenant a ce groupe
    for(Int_t it=0;it<nt;it++)
     {
     valTS *vec=vPtrig[it];
     Int_t nbv=vec->size();
     Int_t i=0;
     while(i < nbv)
      {
      if((*vec)[i][2] == numG) // si c'est un element du groupe
       {
       vec->erase((*vec).begin()+i); // on l'efface
       nbv--;
       }
      else
       {
       i++; // sinon on passe a la valaur suivante
       }
      }
     }
    // On efface aussi les valeurs esclaves inferieures a tcMin si c'est le premier goupe
    //cout << "        Destruction des esclave ? " << ig << " / " << tcMin << endl;
    if(ig==0)
     {
     for(Int_t is=0;is<ns;is++)
      {
      valTS *vec=vPslave[is];
      Int_t nbs=vec->size();
      Int_t i=0;
      while(i < nbs)
       {
       if((*vec)[i][1] < tcMin)
        {
        vec->erase((*vec).begin()+i);
        nbs--;
        }
       else
        {
        i++;
        }
       }
      }
     }
    }
   }
  }
 
 //TString hName=GetName();
 //if(hName=="ThkThn4") cin.get();

 //cout << "Fin Process de " << GetName() << endl;
 //cout << "------------------------------------------------------" << endl;
} 

//____________________________________________________________________
void DataMergerGroup::Reset(void)
{
//
// Initialise les vecteurs
//

} 

//____________________________________________________________________
void DataMergerGroup::ls(Option_t *option) const
{
//
// Affiche le contenu du DataMergerGroup
//
cout << "=============================================================" << endl;
cout << GetName() << "(" << ClassName() << ") : " << endl;
cout << "Triggers : " << pTrigger.size() << endl;
for(Int_t i=0;i<pTrigger.size();i++)
 {
 cout << i+1 << " --> " <<pTrigger[i]->GetName() << " / " << 
 pTriggerTS[i]->GetName() << endl;
 }
cout << "-------------------------------------------------------------" << endl;
cout << "Esclaves : " << pSlave.size() << endl;
for(Int_t i=0;i<pSlave.size();i++)
 {
 cout << i+1 << " --> " << pSlave[i]->GetName() << " / " <<
 pSlaveTS[i]->GetName() << endl;
 }
cout << "-------------------------------------------------------------" << endl;
cout << "Parametres de sortie : " << endl;
for(Int_t i=0;i<pTrigger.size();i+=2)
 {
 cout << i << " --> " << vOutTrig[i]->GetName() << " / " <<
 vOutTrigTS[i]->GetName() << endl;
 }
for(Int_t i=0;i<pSlave.size();i+=2)
 {
 cout << i+nt << " --> " << vOutSlave[i]->GetName() << " / " <<
 vOutSlaveTS[i]->GetName() << endl;
 }
cout << "=============================================================" << endl;

}

//____________________________________________________________________
void DataMergerGroup::PrintValues(void)
{
//
// Affiche le contenu des vecteurs de donnees
//
 cout.precision(25);
 for(Int_t it=0;it<nt;it++)
  {
  cout << "Trigger " << it << " (" <<pTrigger[it]->GetName() << ") : " << (*vPtrig[it]).size() << " valeurs." << endl;
  for(Int_t iv=0;iv<(*vPtrig[it]).size();iv++)
   {
   cout << "     " << iv << " : " << (*vPtrig[it])[iv][0] << " / " << (*vPtrig[it])[iv][1]<< " / grp" << (*vPtrig[it])[iv][2];
   if(iv>0) cout << ", Delta Prec : " << (*vPtrig[it])[iv][1]-(*vPtrig[it])[iv-1][1];
   Double_t nGrp=(*vPtrig[it])[iv][2];
   Int_t ig=0;
   while(ig<vGroupe.size() && nGrp!=vGroupe[ig][0]) ig++;
   cout << ", Delta Groupe : " << (*vPtrig[it])[iv][1]-vGroupe[ig][1] << endl;
   }
  }
 
 for(Int_t is=0;is<ns;is++)
  {
  cout << "Slave " << is << " (" <<pSlave[is]->GetName()<< ") : " << (*vPslave[is]).size() << " valeurs." << endl;
  for(Int_t iv=0;iv<(*vPslave[is]).size();iv++)
   {
   cout << "     " << iv << " : " << (*vPslave[is])[iv][0] << " / " << (*vPslave[is])[iv][1];
   if(iv>0) cout << ", Delta Prec : " << (*vPslave[is])[iv][1]-(*vPslave[is])[iv-1][1];
   for(Int_t ig=0;ig<vGroupe.size();ig++)
    {
    Double_t ngrp=vGroupe[ig][0];
    Double_t tgMin=vGroupe[ig][1];
    cout << ", DeltaT grp" << ngrp << " : "<<  (*vPslave[is])[iv][1]-tgMin;
    }
   cout << endl;
   }
  }
 
 cout << "TS max " << tsMax << endl;
}

#endif
