//
// Classe DataMergerGroup qui permet de regrouper les donnees
// apparaissant dans une fenetre en temps donnee
// Cette version fonctionne avec 1 parametre declencheur
// et des parametres esclaves
//

#ifndef DataMergerGroup_h
#define DataMergerGroup_h

#include "TObject.h"
#include "FParam.h"
#include <vector>

class DataMergerGroup : public TNamed
{
 private:
    Double_t tsMin;
    Int_t    itMin;
    Int_t    nt;
    Int_t    ns;
    vector<Int_t> nbTrig;
    ULong_t  nbGroupes;
    
 protected:
    virtual Bool_t Decision(Int_t ig)=0;
    virtual Double_t ComputeTriggerTime(Int_t ig);
    virtual void MergeData(Int_t ig);
    virtual void PrintValues(void);
 
 
 protected:
    typedef vector<Double_t *> valTS;
    
    Double_t cDelay;
    Double_t cGate;

    Double_t dGate;
    
    Double_t chanToNs;
    Double_t tsMax;
    
    vector<FParam *> pTrigger;
    vector<FParam *> pTriggerTS;
    
    vector<FParam *> pSlave;
    vector<FParam *> pSlaveTS;
    
    vector<FParam *> vOutTrig;
    vector<FParam *> vOutTrigTS;
    vector<FParam *> vOutSlave;
    vector<FParam *> vOutSlaveTS;
    
    vector<valTS *>  vPtrig;
    vector<valTS *>  vPslave;
    
    valTS  vGroupe;
    
 public:
     DataMergerGroup(void);
     ~DataMergerGroup(void);
     
     virtual void SetCollectionDelay(Double_t del) {cDelay=del;}
     virtual void SetCollectionGate(Double_t del) {cGate=del;}
     virtual Double_t GetCollectionDelay(void) {return cDelay;}
     virtual Double_t GetCollectionGate(void) {return cGate;}
     virtual void SetDecisionGate(Double_t del) {dGate=del;}
     virtual Double_t GetDecisionGate(void) {return dGate;}
     virtual void SetTSMax(Double_t del) {tsMax=del;}
     virtual Double_t GetTSMax(void) {return tsMax;}
     
     virtual void Reset(void);
     virtual void AddTriggerParam(FParam *px, FParam *TSx,
                                  FParam *&opx, FParam *&oTSx);
     virtual void AddSlaveParam(FParam *ps, FParam *TSs,
                                FParam *&ops, FParam *&oTSs);
     virtual void Process(void); 
     virtual void ls(Option_t *option="") const;

//ClassDef(DataMergerGroup,1)
};

#endif
