// 
// Patron pour la generation d'une classe derivee dans ROOT
// FNetHistoPlotter derive de TNamed
// 
 
#ifndef FNetHistoPlotter_h 
#define FNetHistoPlotter_h 
 
#include "TNamed.h" 
#include "FNetConnection.h" 
#include "TList.h" 
#include "TMessage.h" 
#include "TTimer.h" 
#include "TH1.h" 
#include "TCutG.h"
 
class FNetHistoPlotter : public FNetConnection, public TQObject { 
 
   protected: 
	static Int_t Debug;

	TList   *listOfHistos;
	Int_t	autoRefresh;
	TTimer	*timer;
   
   protected: 
	virtual void Init(void);       // initialisation 
 
   public:	 
        static void SetDebug(Int_t d=0) {FNetHistoPlotter::Debug=d;}
        static Int_t GetDebug(void) {return FNetHistoPlotter::Debug;}
	
   	FNetHistoPlotter(void);    // constructeur par defaut 
	FNetHistoPlotter(const FNetHistoPlotter &ap); // constructeur par copie 
	
	~FNetHistoPlotter(void);		// desctructeur; 
	 
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	FNetHistoPlotter& operator = (const FNetHistoPlotter &hp); // operateur = 
 
 	virtual void InitSocket(const Char_t *ser=0, Int_t port=0);
	virtual void ReleaseSocket(void);
	virtual void GetListFromServer(void);
	virtual TList *GetListOfObjects(void) const {return listOfHistos;}
	virtual void ListObjects(void);
	virtual TObject *GetObjectFromList(Char_t *nom);
	virtual TObject *GetObjectFromList(Int_t index);
	virtual TH1 *DrawHisto(Char_t *nom,Option_t *opt="");
	virtual void SetAutoRefresh(Int_t s=0);
	virtual Int_t GetAutoRefresh(void)const {return autoRefresh;}
	virtual Bool_t SameHisto(TH1 *old, TH1 *n);
	virtual void UpdateHistoContent(TH1 *old, TH1 *n);
	virtual void ListRefreshed(void) {Emit("ListRefreshed()");} // *SIGNAL*
	virtual TCutG *BuildTCutG(Char_t const *cutName="CutG",
				  Bool_t sendToServer=kFALSE);
	
    ClassDef(FNetHistoPlotter,1)
 
}; 
 
#endif 
 
